#include "common.h"
#include "infrared_gun.h"

int infrared_Init() {
    pinMode(IR_PIN_rec, OUTPUT);  // 设置IR_PIN为输出模式
    pinMode(IR_PIN, INPUT);  // 设置IR_PIN为输入模式

    return 0;
}

// 发送NEC协议的数据
void sendNEC(uint32_t data) {
    int i;
    
    // 发送起始码
    digitalWrite(IR_PIN_rec, HIGH);
    Delay_us(START_PULSE_HIGH);   // 起始码高电平
    digitalWrite(IR_PIN_rec, LOW);
    Delay_us(START_PULSE_LOW);    // 起始码低电平
    
    // 发送数据32位（包括地址、命令和校验位）
    for (i = 31; i >= 0; i--) {
        if ((data >> i) & 1) {
            // 发送逻辑1
            digitalWrite(IR_PIN_rec, HIGH);
            Delay_us(ONE_PULSE_HIGH);  // 高电平Delay_us(int usec)
            digitalWrite(IR_PIN_rec, LOW);
            Delay_us(ONE_PULSE_LOW);   // 低电平
        } else {
            // 发送逻辑0
            digitalWrite(IR_PIN, HIGH);
            Delay_us(ZERO_PULSE_HIGH); // 高电平
            digitalWrite(IR_PIN, LOW);
            Delay_us(ZERO_PULSE_LOW);  // 低电平
        }
    }
    
    // 发送停止码
    digitalWrite(IR_PIN, LOW);
    Delay_us(STOP_PULSE);         // 停止码低电平
}

void receiveNEC() {
    int i;
    uint32_t data = 0;
    
    // 等待起始码
    while (digitalRead(IR_PIN) == LOW);  // 等待高电平
    usleep(START_PULSE_MIN);  // 等待起始码的高电平持续时间（9ms）
    
    // 读取32位数据
    for (i = 31; i >= 0; i--) {
        while (digitalRead(IR_PIN) == LOW);  // 等待高电平
        usleep(ZERO_PULSE_MIN);  // 等待最小高电平时间（0.5ms）
        
        // 检查高电平持续时间来判断是逻辑0还是逻辑1
        if (digitalRead(IR_PIN) == HIGH) {
            // 逻辑1
            usleep(ONE_PULSE_MIN);  // 等待逻辑1的持续时间
            data |= (1 << i);
        } else {
            // 逻辑0
            usleep(ZERO_PULSE_MIN);  // 等待逻辑0的持续时间
        }
    }
    
    my_zlog_notice("Received NEC data: 0x%08X\n", data);
}